﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.Data.Services.Client;

namespace AdnAzureUtilities
{
    public class AzureTableHelper
    {
        CloudStorageAccount _account;

        CloudTableClient _tableClient;

        public AzureTableHelper(string accountKeyName, bool useLocalStorage)
        {
            if (useLocalStorage)
            {
                _account = CloudStorageAccount.DevelopmentStorageAccount;
            }
            else
            {
                // Retrieve storage account from connection-string
                string accountkey = CloudConfigurationManager.GetSetting(accountKeyName);

                _account = CloudStorageAccount.Parse(accountkey);
            }

            // Create the table client
            _tableClient = _account.CreateCloudTableClient();

            _tableClient.RetryPolicy = RetryPolicies.Retry(4, TimeSpan.Zero);
        }

        // List Tables.
        // Return true on success, false if not found, throw exception on error.

        public bool ListTables(out List<string> tableList)
        {
            tableList = new List<string>();

            try
            {
                IEnumerable<string> tables = _tableClient.ListTables();

                if (tables != null)
                {
                    tableList.AddRange(tables);
                }

                return true;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
        }

        // Create Table.
        // Return true on success, false if already exists, throw exception on error.

        public bool CreateTable(string tableName)
        {
            try
            {
                _tableClient.CreateTable(tableName);

                return true;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.Conflict)
                    return false;

                throw;
            }
        }

        // Delete Table.
        // Return true on success, false if not found, throw exception on error.

        public bool DeleteTable(string tableName)
        {
            try
            {
                _tableClient.DeleteTable(tableName);

                return true;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
        }

        // Insert entity.
        // Return true on success, false if not found, throw exception on error.

        public bool InsertEntity(string tableName, TableServiceEntity entity)
        {
            try
            {
                TableServiceContext tableServiceContext = 
                    _tableClient.GetDataServiceContext();

                tableServiceContext.AddObject(tableName, entity);
                tableServiceContext.SaveChanges();

                return true;
            }
            catch (DataServiceRequestException ex)
            {
                return false;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
        }

        // Retrieve an entity.
        // Return true on success, false if not found, throw exception on error.

        public bool GetEntity<T>(
            string tableName, 
            string partitionKey, 
            string rowKey,
            out T entity) where T : TableServiceEntity
        {
            entity = null;

            try
            {
                IEnumerable<T> entities =
                 this.QueryEntities<T>
                  (tableName).Where(e => e.PartitionKey == partitionKey && e.RowKey == rowKey).
                      AsTableServiceQuery<T>();

                entity = entities.FirstOrDefault();

                return true;
            }
            catch (DataServiceRequestException)
            {
                return false;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
            catch
            {
                return false;
            }
        }

        // Query entities. Use LINQ clauses to filter data.
        // Return true on success, false if not found, throw exception on error.

        public DataServiceQuery<T> QueryEntities<T>(string tableName) 
            where T : TableServiceEntity
        {
            TableServiceContext tableServiceContext = 
                _tableClient.GetDataServiceContext();

            return tableServiceContext.CreateQuery<T>(tableName);
        }

        // Replace Update entity. Completely replace previous entity with new entity.
        // Return true on success, false if not found, throw exception on error.

        public bool ReplaceUpdateEntity<T>(
            string tableName, 
            string partitionKey, 
            string rowKey, 
            T obj) where T : TableServiceEntity
        {
            try
            {
                TableServiceContext tableServiceContext =
                    _tableClient.GetDataServiceContext();

                IEnumerable<T> entities =
                    this.QueryEntities<T>
                     (tableName).Where(e => e.PartitionKey == partitionKey && e.RowKey == rowKey).
                         AsTableServiceQuery<T>();

                T entity = entities.FirstOrDefault();

                PropertyInfo[] pi = obj.GetType().GetProperties();

                foreach (PropertyInfo p in pi)
                {
                    p.SetValue(entity, p.GetValue(obj, null), null);
                }

                tableServiceContext.AttachTo(tableName, entity);
                tableServiceContext.UpdateObject(entity);
                tableServiceContext.SaveChanges(SaveChangesOptions.ReplaceOnUpdate);

                return true;
            }
            catch (DataServiceRequestException)
            {
                return false;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
        }

        // Merge update an entity (preserve previous properties not overwritten).
        // Return true on success, false if not found, throw exception on error.

        public bool MergeUpdateEntity<T>(
            string tableName, 
            string partitionKey, 
            string rowKey, 
            T obj) where T : TableServiceEntity
        {
            try
            {
                TableServiceContext tableServiceContext = 
                    _tableClient.GetDataServiceContext();

                IEnumerable<T> entities =
                    this.QueryEntities<T>
                     (tableName).Where(e => e.PartitionKey == partitionKey && e.RowKey == rowKey).
                         AsTableServiceQuery<T>();

                T entity = entities.FirstOrDefault();

                PropertyInfo[] pi =  obj.GetType().GetProperties();

                foreach (PropertyInfo p in pi)
                {
                    p.SetValue(entity, p.GetValue(obj, null), null);
                }

                tableServiceContext.UpdateObject(entity);
                tableServiceContext.SaveChanges();

                return true;
            }
            catch (DataServiceRequestException)
            {
                return false;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
        }

        // Delete entity.
        // Return true on success, false if not found, throw exception on error.

        public bool DeleteEntity<T>(
            string tableName, 
            string partitionKey, 
            string rowKey) where T : TableServiceEntity
        {
            try
            {
                TableServiceContext tableServiceContext = 
                    _tableClient.GetDataServiceContext();

                IEnumerable<T> entities =
                     this.QueryEntities<T>
                      (tableName).Where(e => e.PartitionKey == partitionKey && e.RowKey == rowKey).
                          AsTableServiceQuery<T>();

                T entity = entities.FirstOrDefault();

                if (entity != null)
                {
                    tableServiceContext.Detach(entity);

                    tableServiceContext.AttachTo(tableName, entity, "*");

                    tableServiceContext.DeleteObject(entity);

                    tableServiceContext.SaveChanges(SaveChangesOptions.ReplaceOnUpdate);

                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (DataServiceRequestException ex)
            {
                return false;
            }
            catch (StorageClientException ex)
            {
                if (ex.StatusCode == System.Net.HttpStatusCode.NotFound)
                    return false;

                throw;
            }
        }
    }
}


