﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using Microsoft.WindowsAzure.StorageClient;
using System.ServiceModel.Activation;
using AdnAzureUtilities;
using Newtonsoft.Json;

namespace MaterialWebRole
{
    /////////////////////////////////////////////////////////////////////////////
    // MaterialTableServiceEntity
    // Defines the kind of entities we store in our Azure Table
    //
    /////////////////////////////////////////////////////////////////////////////
    public class MaterialTableServiceEntity : TableServiceEntity
    {
        public readonly static string InfoPartitionKey = "AdnMaterialSrvKey";

        public MaterialTableServiceEntity()
            : base()
        {

        }

        public MaterialTableServiceEntity(AdnMaterial properties) :
            base(InfoPartitionKey, properties.Name)
        {
            Name = properties.Name;

            Price = properties.Price;

            Manufacturer = properties.Manufacturer;
        }

        public AdnMaterial ToMaterialProperties()
        {
            return new AdnMaterial(Name, Price, Manufacturer);
        }

        public string Name
        {
            get;
            set;
        }

        public double Price
        {
            get;
            set;
        }

        public string Manufacturer
        {
            get;
            set;
        }
    }

    /////////////////////////////////////////////////////////////////////////////
    // Implementation of our Web Service
    //
    /////////////////////////////////////////////////////////////////////////////
    [AspNetCompatibilityRequirements(RequirementsMode = 
        AspNetCompatibilityRequirementsMode.Allowed)]
    public class MaterialSrv : IMaterialSrv
    {
        private string _tableName = "adnmaterialsrvtable";

        private AzureTableHelper _tableHelper;

        /////////////////////////////////////////////////////////////////////////
        // Ŝervice constructor: by default invoked at every call of a member
        //
        /////////////////////////////////////////////////////////////////////////
        public MaterialSrv()
        {
            try
            {
                _tableHelper = new AzureTableHelper("adnstorage", false);

                //_tableHelper.DeleteTable(_tableName);

                _tableHelper.CreateTable(_tableName);
            }
            catch (Exception ex)
            {
                //System.Windows.Forms.MessageBox.Show(ex.Message);
            }
        }

        /////////////////////////////////////////////////////////////////////////
        // Returns list of all existing materials
        //
        /////////////////////////////////////////////////////////////////////////
        public AdnMaterial[] GetMaterials()
        {
            WebOperationContext ctx = WebOperationContext.Current;

            try
            {
                List<AdnMaterial> materials = new List<AdnMaterial>();

                IEnumerable<MaterialTableServiceEntity> entities =
                 _tableHelper.QueryEntities<MaterialTableServiceEntity>
                    (_tableName).Where(e => e.PartitionKey == MaterialTableServiceEntity.InfoPartitionKey).
                        AsTableServiceQuery<MaterialTableServiceEntity>();

                if (entities != null)
                {
                    foreach (MaterialTableServiceEntity entity in entities)
                    {
                        materials.Add(entity.ToMaterialProperties());
                    }
                }

                ctx.OutgoingResponse.StatusCode = 
                    System.Net.HttpStatusCode.OK;

                return materials.ToArray();
            }
            catch (Exception ex)
            {
                ctx.OutgoingResponse.StatusCode = 
                    System.Net.HttpStatusCode.NoContent;

                return null;
            }
        }

        /////////////////////////////////////////////////////////////////////////
        // Returns a specific material based on provided name
        //
        /////////////////////////////////////////////////////////////////////////
        public AdnMaterial GetMaterial(string materialName)
        {
            WebOperationContext ctx = WebOperationContext.Current;

            try
            {
                MaterialTableServiceEntity entity = null;

                _tableHelper.GetEntity<MaterialTableServiceEntity>(
                    _tableName,
                    MaterialTableServiceEntity.InfoPartitionKey,
                    materialName,
                    out entity);

                if (entity != null)
                    return entity.ToMaterialProperties();

                AdnMaterial newProps =
                    new AdnMaterial(materialName, 1.0, "ADN");

                bool res = _tableHelper.InsertEntity(
                    _tableName,
                    new MaterialTableServiceEntity(newProps));

                ctx.OutgoingResponse.StatusCode = 
                    System.Net.HttpStatusCode.OK;

                return newProps;
            }
            catch (Exception ex)
            {
                ctx.OutgoingResponse.StatusCode = 
                    System.Net.HttpStatusCode.NotFound;

                return null;
            }
        }

        /////////////////////////////////////////////////////////////////////////
        // Upload new material information for existing material
        //
        /////////////////////////////////////////////////////////////////////////
        public bool PostMaterial(AdnMaterial material)
        {
            WebOperationContext ctx = WebOperationContext.Current;

            try
            {
                MaterialTableServiceEntity entity = 
                    new MaterialTableServiceEntity(material);

                _tableHelper.ReplaceUpdateEntity<MaterialTableServiceEntity>(
                    _tableName,
                    MaterialTableServiceEntity.InfoPartitionKey,
                    material.Name,
                    entity);

                ctx.OutgoingResponse.StatusCode = 
                    System.Net.HttpStatusCode.OK;

                return true;
            }
            catch (Exception ex)
            {
                ctx.OutgoingResponse.StatusCode = 
                    System.Net.HttpStatusCode.BadRequest;

                return false;
            }
        }

        /////////////////////////////////////////////////////////////////////////
        // Deletes a specific material based on provided name
        //
        /////////////////////////////////////////////////////////////////////////
        public bool DeleteMaterial(string materialName)
        {
            WebOperationContext ctx = WebOperationContext.Current;

            try
            {
                _tableHelper.DeleteEntity<MaterialTableServiceEntity>(
                    _tableName,
                    MaterialTableServiceEntity.InfoPartitionKey,
                    materialName);

                ctx.OutgoingResponse.StatusCode =
                   System.Net.HttpStatusCode.OK;

                return true;
            }
            catch (Exception ex)
            {
                ctx.OutgoingResponse.StatusCode =
                   System.Net.HttpStatusCode.NotFound;

                return false;
            }
        }
    }
}
